@@ -7,7 +7,6 @@ use async_trait::async_trait;
 use futures::channel::oneshot;
 use futures::task::SpawnExt;
 use futures::Future;
-use rand::{rngs::StdRng, CryptoRng, Rng, SeedableRng};
 use std::convert::TryInto;
 use std::sync::{
     atomic::{AtomicU32, Ordering},
@@ -38,30 +37,27 @@ pub(crate) trait Buildable: Sized {
     ///
     /// (Since we don't have a CircTarget here, we can't extend the circuit
     /// to be multihop later on.)
-    async fn create_chantarget<RNG: CryptoRng + Rng + Send, RT: Runtime>(
+    async fn create_chantarget<RT: Runtime>(
         chanmgr: &ChanMgr<RT>,
         rt: &RT,
-        rng: &mut RNG,
         ct: &OwnedChanTarget,
         params: &CircParameters,
     ) -> Result<Self>;
 
     /// Launch a new circuit through a given relay, given a circuit target
     /// `ct` specifying that relay.
-    async fn create<RNG: CryptoRng + Rng + Send, RT: Runtime>(
+    async fn create<RT: Runtime>(
         chanmgr: &ChanMgr<RT>,
         rt: &RT,
-        rng: &mut RNG,
         ct: &OwnedCircTarget,
         params: &CircParameters,
     ) -> Result<Self>;
 
     /// Extend this circuit-like object by one hop, to the location described
     /// in `ct`.
-    async fn extend<RNG: CryptoRng + Rng + Send, RT: Runtime>(
+    async fn extend<RT: Runtime>(
         &self,
         rt: &RT,
-        rng: &mut RNG,
         ct: &OwnedCircTarget,
         params: &CircParameters,
     ) -> Result<()>;
@@ -72,14 +68,13 @@ pub(crate) trait Buildable: Sized {
 ///
 /// This is common code, shared by all the first-hop functions in the
 /// implementation of `Buildable` for `Arc<ClientCirc>`.
-async fn create_common<RNG: CryptoRng + Rng + Send, RT: Runtime, CT: ChanTarget>(
+async fn create_common<RT: Runtime, CT: ChanTarget>(
     chanmgr: &ChanMgr<RT>,
     rt: &RT,
-    rng: &mut RNG,
     target: &CT,
 ) -> Result<PendingClientCirc> {
     let chan = chanmgr.get_or_launch(target).await?;
-    let (pending_circ, reactor) = chan.new_circ(rng).await?;
+    let (pending_circ, reactor) = chan.new_circ().await?;
 
     rt.spawn(async {
         let _ = reactor.run().await;
@@ -89,35 +84,32 @@ async fn create_common<RNG: CryptoRng + Rng + Send, RT: Runtime, CT: ChanTarget>
 }
 
 #[async_trait]
-impl Buildable for Arc<ClientCirc> {
-    async fn create_chantarget<RNG: CryptoRng + Rng + Send, RT: Runtime>(
+impl Buildable for ClientCirc {
+    async fn create_chantarget<RT: Runtime>(
         chanmgr: &ChanMgr<RT>,
         rt: &RT,
-        rng: &mut RNG,
         ct: &OwnedChanTarget,
         params: &CircParameters,
     ) -> Result<Self> {
-        let circ = create_common(chanmgr, rt, rng, ct).await?;
-        Ok(circ.create_firsthop_fast(rng, params).await?)
+        let circ = create_common(chanmgr, rt, ct).await?;
+        Ok(circ.create_firsthop_fast(params).await?)
     }
-    async fn create<RNG: CryptoRng + Rng + Send, RT: Runtime>(
+    async fn create<RT: Runtime>(
         chanmgr: &ChanMgr<RT>,
         rt: &RT,
-        rng: &mut RNG,
         ct: &OwnedCircTarget,
         params: &CircParameters,
     ) -> Result<Self> {
-        let circ = create_common(chanmgr, rt, rng, ct).await?;
-        Ok(circ.create_firsthop_ntor(rng, ct, params).await?)
+        let circ = create_common(chanmgr, rt, ct).await?;
+        Ok(circ.create_firsthop_ntor(ct, params.clone()).await?)
     }
-    async fn extend<RNG: CryptoRng + Rng + Send, RT: Runtime>(
+    async fn extend<RT: Runtime>(
         &self,
         _rt: &RT,
-        rng: &mut RNG,
         ct: &OwnedCircTarget,
         params: &CircParameters,
     ) -> Result<()> {
-        ClientCirc::extend_ntor(self, rng, ct, params).await?;
+        self.extend_ntor(ct, params).await?;
         Ok(())
     }
 }
@@ -160,13 +152,12 @@ impl<R: Runtime, C: Buildable + Sync + Send + 'static> Builder<R, C> {
     ///
     /// (TODO: Find
     /// a better design there.)
-    async fn build_notimeout<RNG: CryptoRng + Rng + Send>(
+    async fn build_notimeout(
         self: Arc<Self>,
         path: OwnedPath,
         params: CircParameters,
         start_time: Instant,
         n_hops_built: Arc<AtomicU32>,
-        mut rng: RNG,
         guard_status: Arc<GuardStatusHandle>,
     ) -> Result<C> {
         match path {
@@ -174,8 +165,7 @@ impl<R: Runtime, C: Buildable + Sync + Send + 'static> Builder<R, C> {
                 // If we fail now, it's the guard's fault.
                 guard_status.pending(GuardStatus::Failure);
                 let circ =
-                    C::create_chantarget(&self.chanmgr, &self.runtime, &mut rng, &target, &params)
-                        .await?;
+                    C::create_chantarget(&self.chanmgr, &self.runtime, &target, &params).await?;
                 self.timeouts
                     .note_hop_completed(0, self.runtime.now() - start_time, true);
                 n_hops_built.fetch_add(1, Ordering::SeqCst);
@@ -186,8 +176,7 @@ impl<R: Runtime, C: Buildable + Sync + Send + 'static> Builder<R, C> {
                 let n_hops = p.len() as u8;
                 // If we fail now, it's the guard's fault.
                 guard_status.pending(GuardStatus::Failure);
-                let circ =
-                    C::create(&self.chanmgr, &self.runtime, &mut rng, &p[0], &params).await?;
+                let circ = C::create(&self.chanmgr, &self.runtime, &p[0], &params).await?;
                 self.timeouts
                     .note_hop_completed(0, self.runtime.now() - start_time, n_hops == 0);
                 // If we fail after this point, we can't tell whether it's
@@ -196,7 +185,7 @@ impl<R: Runtime, C: Buildable + Sync + Send + 'static> Builder<R, C> {
                 n_hops_built.fetch_add(1, Ordering::SeqCst);
                 let mut hop_num = 1;
                 for relay in p[1..].iter() {
-                    circ.extend(&self.runtime, &mut rng, relay, &params).await?;
+                    circ.extend(&self.runtime, relay, &params).await?;
                     n_hops_built.fetch_add(1, Ordering::SeqCst);
                     self.timeouts.note_hop_completed(
                         hop_num,
@@ -211,11 +200,10 @@ impl<R: Runtime, C: Buildable + Sync + Send + 'static> Builder<R, C> {
     }
 
     /// Build a circuit from an [`OwnedPath`].
-    async fn build_owned<RNG: CryptoRng + Rng + Send + 'static>(
+    async fn build_owned(
         self: &Arc<Self>,
         path: OwnedPath,
         params: &CircParameters,
-        rng: RNG,
         guard_status: Arc<GuardStatusHandle>,
     ) -> Result<C> {
         let action = Action::BuildCircuit { length: path.len() };
@@ -235,7 +223,6 @@ impl<R: Runtime, C: Buildable + Sync + Send + 'static> Builder<R, C> {
             params,
             start_time,
             Arc::clone(&hops_built),
-            rng,
             guard_status,
         );
 
@@ -261,9 +248,9 @@ impl<R: Runtime, C: Buildable + Sync + Send + 'static> Builder<R, C> {
 /// unless you are choosing your own paths.
 pub struct CircuitBuilder<R: Runtime> {
     /// The underlying [`Builder`] object
-    builder: Arc<Builder<R, Arc<ClientCirc>>>,
+    builder: Arc<Builder<R, ClientCirc>>,
     /// Configuration for how to choose paths for circuits.
-    path_config: crate::PathConfig,
+    path_config: tor_config::MutCfg<crate::PathConfig>,
     /// State-manager object to use in storing current state.
     storage: crate::TimeoutStateHandle,
     /// Guard manager to tell us which guards nodes to use for the circuits
@@ -286,12 +273,22 @@ impl<R: Runtime> CircuitBuilder<R> {
 
         CircuitBuilder {
             builder: Arc::new(Builder::new(runtime, chanmgr, timeouts)),
-            path_config,
+            path_config: path_config.into(),
             storage,
             guardmgr,
         }
     }
 
+    /// Return this builder's [`PathConfig`](crate::PathConfig).
+    pub(crate) fn path_config(&self) -> Arc<crate::PathConfig> {
+        self.path_config.get()
+    }
+
+    /// Replace this builder's [`PathConfig`](crate::PathConfig).
+    pub(crate) fn set_path_config(&self, new_config: crate::PathConfig) {
+        self.path_config.replace(new_config);
+    }
+
     /// Flush state to the state manager.
     pub(crate) fn save_state(&self) -> Result<()> {
         // TODO: someday we'll want to only do this if there is something
@@ -328,17 +325,14 @@ impl<R: Runtime> CircuitBuilder<R> {
         self.builder.timeouts.update_params(p);
     }
 
-    /// DOCDOC
-    pub(crate) async fn build_owned<RNG: CryptoRng + Rng + Send + 'static>(
+    /// Like `build`, but construct a new circuit from an [`OwnedPath`].
+    pub(crate) async fn build_owned(
         &self,
         path: OwnedPath,
         params: &CircParameters,
-        rng: RNG,
         guard_status: Arc<GuardStatusHandle>,
-    ) -> Result<Arc<ClientCirc>> {
-        self.builder
-            .build_owned(path, params, rng, guard_status)
-            .await
+    ) -> Result<ClientCirc> {
+        self.builder.build_owned(path, params, guard_status).await
     }
 
     /// Try to construct a new circuit from a given path, using appropriate
@@ -347,21 +341,9 @@ impl<R: Runtime> CircuitBuilder<R> {
     /// This circuit is _not_ automatically registered with any
     /// circuit manager; if you don't hang on it it, it will
     /// automatically go away when the last reference is dropped.
-    pub async fn build<RNG: CryptoRng + Rng>(
-        &self,
-        path: &TorPath<'_>,
-        params: &CircParameters,
-        rng: &mut RNG,
-    ) -> Result<Arc<ClientCirc>> {
-        let rng = StdRng::from_rng(rng).expect("couldn't construct temporary rng");
+    pub async fn build(&self, path: &TorPath<'_>, params: &CircParameters) -> Result<ClientCirc> {
         let owned = path.try_into()?;
-        self.build_owned(owned, params, rng, Arc::new(None.into()))
-            .await
-    }
-
-    /// Return the path configuration used by this builder.
-    pub(crate) fn path_config(&self) -> &crate::PathConfig {
-        &self.path_config
+        self.build_owned(owned, params, Arc::new(None.into())).await
     }
 
     /// Return true if this builder is currently learning timeout info.
@@ -429,6 +411,7 @@ mod test {
     use std::sync::Mutex;
     use tor_llcrypto::pk::ed25519::Ed25519Identity;
     use tor_rtcompat::{test_with_all_runtimes, SleepProvider};
+    use tracing::trace;
 
     /// Make a new nonfunctional `Arc<GuardStatusHandle>`
     fn gs() -> Arc<GuardStatusHandle> {
@@ -455,7 +438,7 @@ mod test {
             assert!(x.is_ok());
             assert_eq!(x.unwrap(), 3_u32);
 
-            eprintln!("acquiesce after test1");
+            trace!("acquiesce after test1");
             #[allow(clippy::clone_on_copy)]
             let rt = tor_rtmock::MockSleepRuntime::new(rto.clone());
 
@@ -467,22 +450,19 @@ mod test {
                 .wait_for(double_timeout(
                     &rt,
                     async move {
-                        dbg!("A");
                         let sl = rt_clone.sleep(Duration::from_millis(100));
                         rt_clone.allow_one_advance(Duration::from_millis(100));
                         sl.await;
-                        dbg!("B");
                         Ok(4_u32)
                     },
                     t1,
                     t10,
                 ))
                 .await;
-            dbg!(&x);
             assert!(x.is_ok());
             assert_eq!(x.unwrap(), 4_u32);
 
-            eprintln!("acquiesce after test2");
+            trace!("acquiesce after test2");
             #[allow(clippy::clone_on_copy)]
             let rt = tor_rtmock::MockSleepRuntime::new(rto.clone());
 
@@ -512,7 +492,7 @@ mod test {
             let waited = rt.wait_for(rcv).await;
             assert_eq!(waited, Ok(()));
 
-            eprintln!("acquiesce after test3");
+            trace!("acquiesce after test3");
             #[allow(clippy::clone_on_copy)]
             let rt = tor_rtmock::MockSleepRuntime::new(rto.clone());
 
@@ -543,7 +523,6 @@ mod test {
             assert!(waited.is_err());
             let end2 = rt.now();
             assert!(duration_close_to(end - start, Duration::from_secs(1)));
-            dbg!(end2, start, end2 - start);
             assert!(duration_close_to(end2 - start, Duration::from_secs(10)));
         });
     }
@@ -589,10 +568,9 @@ mod test {
     }
     #[async_trait]
     impl Buildable for Mutex<FakeCirc> {
-        async fn create_chantarget<RNG: CryptoRng + Rng + Send, RT: Runtime>(
+        async fn create_chantarget<RT: Runtime>(
             _: &ChanMgr<RT>,
             rt: &RT,
-            _: &mut RNG,
             ct: &OwnedChanTarget,
             _: &CircParameters,
         ) -> Result<Self> {
@@ -609,10 +587,9 @@ mod test {
             };
             Ok(Mutex::new(c))
         }
-        async fn create<RNG: CryptoRng + Rng + Send, RT: Runtime>(
+        async fn create<RT: Runtime>(
             _: &ChanMgr<RT>,
             rt: &RT,
-            _: &mut RNG,
             ct: &OwnedCircTarget,
             _: &CircParameters,
         ) -> Result<Self> {
@@ -629,10 +606,9 @@ mod test {
             };
             Ok(Mutex::new(c))
         }
-        async fn extend<RNG: CryptoRng + Rng + Send, RT: Runtime>(
+        async fn extend<RT: Runtime>(
             &self,
             rt: &RT,
-            _: &mut RNG,
             ct: &OwnedCircTarget,
             _: &CircParameters,
         ) -> Result<()> {
@@ -747,13 +723,12 @@ mod test {
             timeouts::Estimator::new(Arc::clone(&timeouts)),
         );
 
-        let rng = StdRng::from_rng(rand::thread_rng()).expect("couldn't construct temporary rng");
         let params = CircParameters::default();
 
         rt.block_advance("manually controlling advances");
         rt.allow_one_advance(advance_initial);
         let outcome = rt
-            .wait_for(Arc::new(builder).build_owned(path, &params, rng, gs()))
+            .wait_for(Arc::new(builder).build_owned(path, &params, gs()))
             .await;
 
         // Now we wait for a success to finally, finally be reported.
@@ -785,7 +760,7 @@ mod test {
             assert!(timeouts[0].0); // success
             assert_eq!(timeouts[0].1, 0); // one-hop
             assert_eq!(timeouts[0].2, Duration::from_millis(100));
-        })
+        });
     }
 
     #[test]
@@ -810,7 +785,7 @@ mod test {
             assert!(timeouts[0].0); // success
             assert_eq!(timeouts[0].1, 2); // three-hop
             assert_eq!(timeouts[0].2, Duration::from_millis(600));
-        })
+        });
     }
 
     #[test]
@@ -861,7 +836,6 @@ mod test {
             .await;
             assert!(matches!(outcome, Err(Error::CircTimeout)));
 
-            dbg!(&timeouts);
             assert_eq!(timeouts.len(), 2);
             assert!(!timeouts[0].0); // timeout
 
