@@ -41,6 +41,14 @@ pub enum Error {
     #[error("{0}")]
     RequestFailed(RetryError<Box<Error>>),
 
+    /// A circuit succeeded, but was cancelled before it could be used.
+    ///
+    /// Circuits can be cancelled either by a call to
+    /// `retire_all_circuits()`, or by a configuration change that
+    /// makes old paths unusable.
+    #[error("Circuit cancelled")]
+    CircCancelled,
+
     /// An error caused by a programming issue or a failure in another
     /// library that we can't work around.
     #[error("Internal error: {0}")]
@@ -65,6 +73,10 @@ pub enum Error {
     /// Problem selecting a guard relay.
     #[error("Unable to select a guard relay: {0}")]
     Guard(#[from] tor_guardmgr::PickGuardError),
+
+    /// We have an expired consensus
+    #[error("Consensus is expired")]
+    ExpiredConsensus,
 }
 
 impl From<futures::channel::oneshot::Canceled> for Error {
