@@ -3,8 +3,8 @@
 use super::TorPath;
 use crate::{DirInfo, Error, PathConfig, Result, TargetPort};
 use rand::Rng;
+use std::time::{Duration, SystemTime};
 use tor_guardmgr::{GuardMgr, GuardMonitor, GuardUsable};
-use tor_linkspec::ChanTarget;
 use tor_netdir::{NetDir, Relay, SubnetConfig, WeightRole};
 use tor_rtcompat::Runtime;
 
@@ -75,7 +75,7 @@ impl<'a> ExitPathBuilder<'a> {
         rng: &mut R,
         netdir: &'a NetDir,
         guard: Option<&Relay<'a>>,
-        config: &SubnetConfig,
+        config: SubnetConfig,
     ) -> Result<Relay<'a>> {
         match &self.inner {
             ExitPathBuilderInner::AnyExit { strict } => {
@@ -126,7 +126,13 @@ impl<'a> ExitPathBuilder<'a> {
             DirInfo::Fallbacks(_) => return Err(Error::NeedConsensus),
             DirInfo::Directory(d) => d,
         };
-        let subnet_config = &config.enforce_distance;
+        let subnet_config = config.subnet_config();
+        let lifetime = netdir.lifetime();
+
+        // Check if the consensus isn't expired by > 72 hours
+        if SystemTime::now() > lifetime.valid_until() + Duration::new(72 * 60 * 60, 0) {
+            return Err(Error::ExpiredConsensus);
+        }
 
         let chosen_exit = if let ExitPathBuilderInner::ChosenExit(e) = &self.inner {
             Some(e)
@@ -143,11 +149,11 @@ impl<'a> ExitPathBuilder<'a> {
                 b.kind(tor_guardmgr::GuardUsageKind::Data);
                 guardmgr.update_network(netdir); // possibly unnecessary.
                 if let Some(exit_relay) = chosen_exit {
-                    // TODO Problem! This doesn't actually enforce a
-                    // distinct family for the guard and the exit.  It
-                    // just makes sure they're not the same relay.
-                    let id = exit_relay.ed_identity();
-                    b.restriction(tor_guardmgr::GuardRestriction::AvoidId(*id));
+                    let mut family = std::collections::HashSet::new();
+                    family.insert(*exit_relay.id());
+                    // TODO(nickm): See "limitations" note on `known_family_members`.
+                    family.extend(netdir.known_family_members(exit_relay).map(|r| *r.id()));
+                    b.push_restriction(tor_guardmgr::GuardRestriction::AvoidAllIds(family));
                 }
                 let guard_usage = b.build().expect("Failed while building guard usage!");
                 let (guard, mut mon, usable) = guardmgr.select_guard(guard_usage, Some(netdir))?;
@@ -194,16 +200,12 @@ impl<'a> ExitPathBuilder<'a> {
 }
 
 /// Returns true if both relays can appear together in the same circuit.
-fn relays_can_share_circuit(a: &Relay<'_>, b: &Relay<'_>, subnet_config: &SubnetConfig) -> bool {
-    // XXX: features missing from original implementation:
-    // - option NodeFamilySets
-    // see: src/feature/nodelist/nodelist.c:nodes_in_same_family()
-
-    !a.in_same_family(b) && !a.in_same_subnet(b, subnet_config)
+fn relays_can_share_circuit(a: &Relay<'_>, b: &Relay<'_>, subnet_config: SubnetConfig) -> bool {
+    !a.in_same_family(b) && !a.in_same_subnet(b, &subnet_config)
 }
 
 /// Helper: wraps relays_can_share_circuit but takes an option.
-fn relays_can_share_circuit_opt(r1: &Relay<'_>, r2: Option<&Relay<'_>>, c: &SubnetConfig) -> bool {
+fn relays_can_share_circuit_opt(r1: &Relay<'_>, r2: Option<&Relay<'_>>, c: SubnetConfig) -> bool {
     match r2 {
         Some(r2) => relays_can_share_circuit(r1, r2, c),
         None => true,
@@ -213,9 +215,11 @@ fn relays_can_share_circuit_opt(r1: &Relay<'_>, r2: Option<&Relay<'_>>, c: &Subn
 #[cfg(test)]
 mod test {
     #![allow(clippy::unwrap_used)]
+    #![allow(clippy::clone_on_copy)]
     use super::*;
     use crate::path::{assert_same_path_when_owned, OwnedPath, TorPathInner};
     use crate::test::OptDummyGuardMgr;
+    use std::collections::HashSet;
     use std::convert::TryInto;
     use tor_linkspec::ChanTarget;
     use tor_netdir::testnet;
@@ -234,9 +238,9 @@ mod test {
         assert!(r2.ed_identity() != r3.ed_identity());
 
         let subnet_config = SubnetConfig::default();
-        assert!(relays_can_share_circuit(r1, r2, &subnet_config));
-        assert!(relays_can_share_circuit(r1, r3, &subnet_config));
-        assert!(relays_can_share_circuit(r2, r3, &subnet_config));
+        assert!(relays_can_share_circuit(r1, r2, subnet_config));
+        assert!(relays_can_share_circuit(r1, r3, subnet_config));
+        assert!(relays_can_share_circuit(r2, r3, subnet_config));
     }
 
     #[test]
@@ -326,4 +330,138 @@ mod test {
         let owned: Result<OwnedPath> = (&bogus_path).try_into();
         assert!(owned.is_err());
     }
+
+    #[test]
+    fn no_exits() {
+        // Construct a netdir with no exits.
+        let netdir = testnet::construct_custom_netdir(|_idx, bld| {
+            bld.md.parse_ipv4_policy("reject 1-65535").unwrap();
+        })
+        .unwrap()
+        .unwrap_if_sufficient()
+        .unwrap();
+        let mut rng = rand::thread_rng();
+        let dirinfo = (&netdir).into();
+        let guards: OptDummyGuardMgr<'_> = None;
+        let config = PathConfig::default();
+
+        // With target ports
+        let outcome = ExitPathBuilder::from_target_ports(vec![TargetPort::ipv4(80)])
+            .pick_path(&mut rng, dirinfo, guards, &config);
+        assert!(outcome.is_err());
+        assert!(matches!(outcome, Err(Error::NoRelays(_))));
+
+        // For any exit
+        let outcome = ExitPathBuilder::for_any_exit().pick_path(&mut rng, dirinfo, guards, &config);
+        assert!(outcome.is_err());
+        assert!(matches!(outcome, Err(Error::NoRelays(_))));
+
+        // For any exit (non-strict, so this will work).
+        let outcome =
+            ExitPathBuilder::for_timeout_testing().pick_path(&mut rng, dirinfo, guards, &config);
+        assert!(outcome.is_ok());
+    }
+
+    #[test]
+    fn exitpath_with_guards() {
+        use tor_guardmgr::GuardStatus;
+
+        tor_rtcompat::test_with_all_runtimes!(|rt| async move {
+            let netdir = testnet::construct_netdir()
+                .unwrap()
+                .unwrap_if_sufficient()
+                .unwrap();
+            let mut rng = rand::thread_rng();
+            let dirinfo = (&netdir).into();
+            let statemgr = tor_persist::TestingStateMgr::new();
+            let guards = tor_guardmgr::GuardMgr::new(rt.clone(), statemgr).unwrap();
+            let config = PathConfig::default();
+            guards.update_network(&netdir);
+            let port443 = TargetPort::ipv4(443);
+
+            // We're going to just have these all succeed and make sure
+            // that they pick the same guard.  We won't test failing
+            // cases here, since those are tested in guardmgr.
+            let mut distinct_guards = HashSet::new();
+            let mut distinct_mid = HashSet::new();
+            let mut distinct_exit = HashSet::new();
+            for _ in 0..20 {
+                let (path, mon, usable) = ExitPathBuilder::from_target_ports(vec![port443])
+                    .pick_path(&mut rng, dirinfo, Some(&guards), &config)
+                    .unwrap();
+                assert_eq!(path.len(), 3);
+                assert_same_path_when_owned(&path);
+                if let TorPathInner::Path(p) = path.inner {
+                    assert_exit_path_ok(&p[..]);
+                    distinct_guards.insert(p[0].ed_identity().clone());
+                    distinct_mid.insert(p[1].ed_identity().clone());
+                    distinct_exit.insert(p[2].ed_identity().clone());
+                } else {
+                    panic!("Wrong kind of path");
+                }
+                let mon = mon.unwrap();
+                assert!(matches!(
+                    mon.inspect_pending_status(),
+                    (GuardStatus::AttemptAbandoned, false)
+                ));
+                mon.succeeded();
+                assert!(usable.unwrap().await.unwrap());
+            }
+            assert_eq!(distinct_guards.len(), 1);
+            assert_ne!(distinct_mid.len(), 1);
+            assert_ne!(distinct_exit.len(), 1);
+
+            let guard_relay = netdir
+                .by_id(distinct_guards.iter().next().unwrap())
+                .unwrap();
+            let exit_relay = netdir.by_id(distinct_exit.iter().next().unwrap()).unwrap();
+
+            // Now we'll try a forced exit that is not the same same as our
+            // actual guard.
+            let (path, mon, usable) = ExitPathBuilder::from_chosen_exit(exit_relay.clone())
+                .pick_path(&mut rng, dirinfo, Some(&guards), &config)
+                .unwrap();
+            assert_eq!(path.len(), 3);
+            if let TorPathInner::Path(p) = path.inner {
+                assert_exit_path_ok(&p[..]);
+                // We get our regular guard and our chosen exit.
+                assert_eq!(p[0].ed_identity(), guard_relay.ed_identity());
+                assert_eq!(p[2].ed_identity(), exit_relay.ed_identity());
+            } else {
+                panic!("Wrong kind of path");
+            }
+            let mon = mon.unwrap();
+            // This time, "ignore indeterminate status" was set to true.
+            assert!(matches!(
+                mon.inspect_pending_status(),
+                (GuardStatus::AttemptAbandoned, true)
+            ));
+            mon.succeeded();
+            assert!(usable.unwrap().await.unwrap());
+
+            // Finally, try with our exit forced to be our regular guard,
+            // and make sure we get a different guard.
+            let (path, mon, usable) = ExitPathBuilder::from_chosen_exit(guard_relay.clone())
+                .pick_path(&mut rng, dirinfo, Some(&guards), &config)
+                .unwrap();
+            assert_eq!(path.len(), 3);
+            if let TorPathInner::Path(p) = path.inner {
+                // This is no longer guaranteed; see arti#183 :(
+                // assert_exit_path_ok(&p[..]);
+                // We get our chosen exit, and a different guard.
+                assert_ne!(p[0].ed_identity(), guard_relay.ed_identity());
+                assert_eq!(p[2].ed_identity(), guard_relay.ed_identity());
+            } else {
+                panic!("Wrong kind of path");
+            }
+            let mon = mon.unwrap();
+            // This time, "ignore indeterminate status" was set to true.
+            assert!(matches!(
+                mon.inspect_pending_status(),
+                (GuardStatus::AttemptAbandoned, true)
+            ));
+            mon.succeeded();
+            assert!(usable.unwrap().await.unwrap());
+        });
+    }
 }
