@@ -136,21 +136,25 @@ impl History {
     where
         I: Iterator<Item = (MsecDuration, u16)>,
     {
-        // XXXX if the input is bogus, then this could be a huge array.
-        let mut observations = Vec::new();
-        for (d, n) in iter {
-            for _ in 0..n {
-                observations.push(d)
-            }
-        }
-        use rand::seq::SliceRandom;
+        use rand::seq::{IteratorRandom, SliceRandom};
+        use std::iter;
         let mut rng = rand::thread_rng();
-        observations[..].shuffle(&mut rng);
+
+        // We want to build a vector with the elements of the old histogram in
+        // random order, but we want to defend ourselves against bogus inputs
+        // that would take too much RAM.
+        let mut observations = iter
+            .take(TIME_HISTORY_LEN) // limit number of bins
+            .flat_map(|(dur, n)| iter::repeat(dur).take(n as usize))
+            .choose_multiple(&mut rng, TIME_HISTORY_LEN);
+        // choose_multiple doesn't guarantee anything about the order of its output.
+        observations.shuffle(&mut rng);
 
         let mut result = History::new_empty();
         for obs in observations {
             result.add_time(obs);
         }
+
         result
     }
 
@@ -176,7 +180,7 @@ impl History {
     /// Increment the histogram bucket containing `time` by one.
     fn inc_bucket(&mut self, time: MsecDuration) {
         let center = History::bucket_center(time);
-        *self.time_histogram.entry(center).or_insert(0) += 1
+        *self.time_histogram.entry(center).or_insert(0) += 1;
     }
 
     /// Decrement the histogram bucket containing `time` by one, removing
@@ -573,10 +577,13 @@ impl super::TimeoutEstimator for ParetoTimeoutEstimator {
         }
     }
 
-    fn note_circ_timeout(&mut self, hop: u8, _delay: Duration) {
-        // XXXXX This only counts if we have recent-enough
-        // activity.  See circuit_build_times_network_check_live.
-        if hop > 0 {
+    fn note_circ_timeout(&mut self, hop: u8, delay: Duration) {
+        // Only record this timeout if we have seen some network activity since
+        // we launched the circuit.
+        let last_traffic = tor_proto::time_since_last_incoming_traffic();
+        let have_seen_recent_activity = last_traffic < delay;
+
+        if hop > 0 && have_seen_recent_activity {
             self.history.add_success(false);
             if self.history.n_recent_timeouts() > self.p.reset_after_timeouts {
                 let base_timeouts = self.base_timeouts();
@@ -610,10 +617,11 @@ impl super::TimeoutEstimator for ParetoTimeoutEstimator {
         let multiplier =
             (action.timeout_scale() as f64) / (reference_action.timeout_scale() as f64);
 
-        // TODO-SPEC The spec define any of self.  Tor doesn't multiply the
+        // TODO-SPEC The spec doesn't define any of this
+        // action-based-multiplier stuff.  Tor doesn't multiply the
         // abandon timeout.
-        // XXXX `mul_f64()` can panic if we overflow Duration.
-        (base_t.mul_f64(multiplier), base_a.mul_f64(multiplier))
+        use super::mul_duration_f64_saturating as mul;
+        (mul(base_t, multiplier), mul(base_a, multiplier))
     }
 
     fn learning_timeouts(&self) -> bool {
